// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include "sim.h"
#include <math.h>

void load_config(struct device *in)
{
int i;
char name[1000];
int ymeshlayers;
FILE *config;

config=fopen("./sim.inp","r");
if (config == NULL)
{
	printf("no file sim.inp\n");
	exit(0);
}

fscanf(config,"%s",name);
fscanf(config,"%s",name);
fclose(config);

config=fopen(name,"r");
if (config == NULL)
{
	printf("no file %s\n",name);
	exit(0);
}
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_ex));

fscanf(config,"%s",name);
fscanf(config,"%d",&ymeshlayers);

struct mesh *meshdata;
meshdata = malloc (ymeshlayers * sizeof(struct mesh));
in->ymeshpoints=0;

for (i=0;i<ymeshlayers;i++)
{
	fscanf(config,"%lf %lf",&(meshdata[i].len),&(meshdata[i].number));
	meshdata[i].den=meshdata[i].len/meshdata[i].number;
	in->ymeshpoints+=meshdata[i].number;
}

in->ymatlayers=malloc (in->ymeshpoints * sizeof(int));

device_init(in);

int pos=0;
int ii;
double dpos=0.0;
for (i=0;i<ymeshlayers;i++)
{

	for (ii=0;ii<meshdata[i].number;ii++)
	{
		dpos+=meshdata[i].den/2.0;
		in->ymesh[pos]=dpos;
		in->ymatlayers[pos]=i;
		dpos+=meshdata[i].den/2.0;
		pos++;
	}
}

fscanf(config,"%d",&(in->mat.number));
in->mat.l = malloc (in->mat.number * sizeof(struct layer));

for (i=0;i<in->mat.number;i++)
{
double temp;
	fscanf(config,"%s %le %le %le",name,&(in->mat.l[i].height),&(in->mat.l[i].Nad),&(temp));
	//printf("year1 %le\n",mat.l[i].Nad);
	load_mat_file(name,&(in->mat.l[i]));
	//printf("year2 \n");
}
//getchar();

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->xlen));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->zlen));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_plot));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->max_electrical_itt));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->electrical_clamp));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_back));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->min_cur_error));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Psun));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstep));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Rbi_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gp_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gn_on));
fclose(config);

}

void device_update_photon_dist(struct device *in)
{
int i;
double a=1e5*100;
struct light light;
read_sun(&light);
double I=light_get_photons(&light,in->Psun,1.0*1.6e-19);
double x=0.0;
//double d=in->ymesh[in->ymeshpoints-1];
	for (i=0;i<in->ymeshpoints;i++)
	{
		x=in->ymesh[i];
		in->Gex[i]=2.7e27;//a*I*exp(-a*x);//+exp(-2*a*d)*exp(a*x); //from blom;//
//printf("%e",in->Gex[i]);
//getchar();

	}

	FILE *out=fopen("./G.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->G[i]);
	}
	fclose(out);
}
