// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>	/* for EXIT_FAILURE */
#include <string.h>
#include "sim.h"
#include <math.h>

void get_initial(struct device *in)
{
printf("get_initial\n");
int i;
double *ln;
double Ef;
ln = malloc(in->ymeshpoints*sizeof(double));
for (i=0;i<in->ymeshpoints;i++)
{
	if (in->Nad[i]>0)	//positave
	{
		ln[i]=-(kb*in->Tl[i]/q)*log(in->Nad[i]/in->Nv[i])-in->Eg[i];		//Fn-Ec
	}else
	{
		ln[i]=(kb*in->Tl[i]/q)*log((-in->Nad[i])/in->Nc[i]);		//
	}
}
Ef=ln[0]-in->Xi[0]-in->phi[0];

//calculate phi
for (i=0;i<in->ymeshpoints;i++)
{
	in->phi[i]=ln[i]-ln[0]-in->Xi[i]+in->Xi[0];

	in->Ec[i]=-in->phi[i]-in->Xi[i];
	in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];
	in->Fi[i]=Ef;
	in->Fn[i]=Ef;
	in->Fp[i]=Ef;


}

for (i=0;i<in->ymeshpoints;i++)
{

	in->n[i]=in->Nc[i]*exp(((in->Fi[i]-in->Ec[i])*q)/(kb*in->Tl[i]));
	in->p[i]=in->Nv[i]*exp(((in->Ev[i]-in->Fi[i])*q)/(kb*in->Tl[i]));
}


//Now get the voltages
in->Vl=in->phi[0];
in->Vr=in->phi[in->ymeshpoints-1];
printf("Vl=%lf Vr=%lf",in->Vl,in->Vr);
//<clean>
FILE *file=fopen("./initial.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
	fprintf(file,"%e %e %e %e %e %e %e %e %e %e %e %e\n",in->ymesh[i],in->Xi[i],in->Eg[i],in->Nad[i],in->n[i],in->p[i],in->phi[i],in->Ec[i],in->Ev[i],Ef,in->Nc[i],in->Nv[i]);
}
fclose(file);
//</clean>

free(ln);
}

