// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void device_set_norm(struct device *in)
{

//<clean>
   in->n0=sqrt(in->Nc[0]*in->Nv[0]);
//   printf("%e %e\n",in->Nc[0],in->Nv[0]);
//	getchar();
   in->x0=sqrt(in->kTq*epsilon0/q/in->n0);

   in->mu0=1e-4;
   in->D0=in->mu0*in->kTq;
   in->t0=in->x0*in->x0/in->D0;
   in->R0=in->n0/in->t0;
   in->Rs0=in->R0;
   in->dR0=in->R0/in->n0;
   in->J0=q*in->n0*in->D0/in->x0;
   in->Dex0=1e30;
	//printf("here=%e %e",in->J0,in->R0);

//getchar();
/*   in->n0=1.0;
//   printf("%e %e\n",in->Nc[0],in->Nv[0]);
//	getchar();
   in->x0=1.0;

   in->mu0=1.0;
   in->D0=1.0;
   in->t0=1.0;
   in->R0=1.0;
   in->Rs0=1.0;
   in->dR0=1.0;
   in->J0=1.0;*/

//</clean>
}

void device_init(struct device *in)
{
	in->phi = malloc(in->ymeshpoints*sizeof(double));
	in->Nad = malloc(in->ymeshpoints*sizeof(double));
	in->n = malloc(in->ymeshpoints*sizeof(double));
	in->p = malloc(in->ymeshpoints*sizeof(double));
	in->ni = malloc(in->ymeshpoints*sizeof(double));
	in->dn = malloc(in->ymeshpoints*sizeof(double));
	in->dp = malloc(in->ymeshpoints*sizeof(double));
	in->Eg = malloc(in->ymeshpoints*sizeof(double));
	in->Fn = malloc(in->ymeshpoints*sizeof(double));
	in->Fp = malloc(in->ymeshpoints*sizeof(double));
	in->Xi = malloc(in->ymeshpoints*sizeof(double));
	in->Ev = malloc(in->ymeshpoints*sizeof(double));
	in->Ec = malloc(in->ymeshpoints*sizeof(double));
	in->mun = malloc(in->ymeshpoints*sizeof(double));
	in->mup = malloc(in->ymeshpoints*sizeof(double));
	in->Nc = malloc(in->ymeshpoints*sizeof(double));
	in->Nv = malloc(in->ymeshpoints*sizeof(double));
	in->mn = malloc(in->ymeshpoints*sizeof(double));
	in->mp = malloc(in->ymeshpoints*sizeof(double));
	in->G = malloc(in->ymeshpoints*sizeof(double));
	in->Gex = malloc(in->ymeshpoints*sizeof(double));
	in->ymesh = malloc (in->ymeshpoints * sizeof(double));
	in->Tl = malloc (in->ymeshpoints * sizeof(double));
	in->Dn = malloc (in->ymeshpoints * sizeof(double));
	in->Dp = malloc (in->ymeshpoints * sizeof(double));
	in->R = malloc (in->ymeshpoints * sizeof(double));
	in->ymatlayers = malloc (in->ymeshpoints * sizeof(int));
	in->Fi = malloc (in->ymeshpoints * sizeof(double));
	in->Jn = malloc (in->ymeshpoints * sizeof(double));
	in->Jp = malloc (in->ymeshpoints * sizeof(double));
	in->x = malloc (in->ymeshpoints * sizeof(double));
	in->t = malloc (in->ymeshpoints * sizeof(double));
	in->xp = malloc (in->ymeshpoints * sizeof(double));
	in->tp = malloc (in->ymeshpoints * sizeof(double));
	in->kf = malloc (in->ymeshpoints * sizeof(double));
	in->a = malloc (in->ymeshpoints * sizeof(double));
	in->Rbi = malloc (in->ymeshpoints * sizeof(double));

	in->back_x = malloc (in->ymeshpoints * sizeof(double));
	in->back_xp = malloc (in->ymeshpoints * sizeof(double));
	in->back_phi = malloc(in->ymeshpoints*sizeof(double));
	in->back_ex = malloc(in->ymeshpoints*sizeof(double));


	in->ex = malloc (in->ymeshpoints * sizeof(double));
	in->Dex = malloc (in->ymeshpoints * sizeof(double));
	in->Hex = malloc (in->ymeshpoints * sizeof(double));
	in->epsilonr = malloc (in->ymeshpoints * sizeof(double));
	solver_get_memory(in);

//	in->gnuplot =start_gnuplot ();

}




void device_free(struct device *in)
{
	free(in->phi);
	free(in->Nad);
	free(in->n);
	free(in->p);
	free(in->ni);
	free(in->dn);
	free(in->dp);
	free(in->Eg);
	free(in->Xi);
	free(in->Ev);
	free(in->Ec);
	free(in->mun);
	free(in->mup);
	free(in->ymesh);
	free(in->Fn);
	free(in->Fp);
	free(in->Nc);
	free(in->Nv);
	free(in->mn);
	free(in->mp);
	free(in->G);
	free(in->Gex);
	free(in->Tl);
	free(in->Dn);
	free(in->Dp);
	free(in->R);
	free(in->ymatlayers);
	free(in->Fi);
	free(in->Jn);
	free(in->Jp);
	free(in->x);
	free(in->t);
	free(in->xp);
	free(in->tp);
	free(in->ex);
	free(in->Dex);
	free(in->Hex);
	free(in->epsilonr);
	free(in->back_x);
	free(in->back_xp);
	free(in->back_phi);
	free(in->back_ex);
	free(in->back_b);
	free(in->b);
	free(in->Ti);
	free(in->Tj);
	free(in->Tx);
	free(in->kf);
	free(in->a);
	free(in->Rbi);
	//fclose(in->gnuplot);




}




double get_error(struct device *in,double *b)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(b[i]);
}
return tot;
}

void update(struct device *cell)
{
int i;
for (i=0;i<cell->ymeshpoints;i++)
{
	//in->Ec[i]=-in->Xi[i]-in->phi[i];
	//in->Ev[i]=-in->Xi[i]-in->Eg[i]-in->phi[i];
	cell->t[i]=cell->Xi[i];
	cell->Ec[i]=-cell->phi[i]-cell->Xi[i];
	cell->Ev[i]=-cell->phi[i]-cell->Xi[i]-cell->Eg[i];
	cell->R[i]=0.0;//1e6;
	cell->Nc[i]=2.0*pow(((2.0*pi*kb*cell->Tl[i]*cell->mn[i]*m0)/(hp*hp)),1.5);
	cell->Nv[i]=2.0*pow(((2.0*pi*kb*cell->Tl[i]*cell->mp[i]*m0)/(hp*hp)),1.5);
	//in->n[i]=in->Nc[i]*exp(((in->Fn[i]-in->Ec[i]))/(kb*in->Tl[i]));
	//in->p[i]=in->Nv[i]*exp(((in->Ev[i]-in->Fp[i]))/(kb*in->Tl[i]));
	cell->dn[i]=q*cell->Nc[i]*exp((q*(cell->Fn[i]-cell->Ec[i]))/(kb*cell->Tl[i]))/(kb*cell->Tl[i]);
	cell->dp[i]=q*cell->Nv[i]*exp((q*(cell->Ev[i]-cell->Fp[i]))/(kb*cell->Tl[i]))/(kb*cell->Tl[i]);
	cell->n[i]=cell->Nc[i]*exp(((cell->Fn[i]-cell->Ec[i])*q)/(kb*cell->Tl[i]));
	cell->p[i]=cell->Nv[i]*exp(((cell->Ev[i]-cell->Fp[i])*q)/(kb*cell->Tl[i]));
	cell->Dn[i]= (kb*cell->Tl[i]/q)*cell->mun[i];
	cell->Dp[i]= (kb*cell->Tl[i]/q)*cell->mup[i];

}

}




void device_dump(struct device *in)
{
int i;
FILE * file=fopen("./out.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
	fprintf(file,"%le %le %le %le %le %le %le %le %le %le %le %le %le\n",in->ymesh[i],in->Nad[i],in->phi[i],in->n[i],in->p[i],in->Nc[i],in->Nv[i],in->Ec[i],in->Ev[i],in->Fn[i],in->Fp[i],in->mun[i],in->mup[i]);
}
fclose(file);
}

int main(void)
{
int i;
struct device cell;
cell.kTq=(300.0*kb/q);

load_config(&cell);
device_update_photon_dist(&cell);
plot_open(&cell);
cell.converge = fopen("./converge.dat","w");
fclose(cell.converge);
int curlayer=0;
double end=cell.mat.l[0].height;

for (i=0;i<cell.ymeshpoints;i++)
{
	if (cell.ymesh[i]>=end)
	{
		curlayer++;
		end+=cell.mat.l[curlayer].height;
	}
	cell.ymatlayers[i]=curlayer;
	cell.Nad[i]=cell.mat.l[cell.ymatlayers[i]].Nad;
	//printf("%d %d %e\n",i,cell.ymatlayers[i],cell.Nad[i]);
}

cell.ylen=0.0;
for (i=0;i<cell.mat.number;i++)
{
cell.ylen+=cell.mat.l[i].height;
}



//printf("Rod=here\n");
	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.Tl[i]=300.0;
		cell.ex[i]=0.0;
		cell.Hex[i]=0.0;
		cell.Dex[i]=100.0;
		if ((i>cell.ymeshpoints/2)&&(i<cell.ymeshpoints/2+10)) cell.Hex[i]=1e9;
		cell.epsilonr[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_epsilonr);
		//printf("0 %d\n",cell.ymatlayers[i]);
		cell.Eg[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Eg);
		cell.Dex[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Dex);
		//printf("%lf\n",cell.Eg[i]);
		//getchar();
		cell.Xi[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Xi);
		//printf("2\n");
		cell.Ec[i]=-cell.Xi[i];
		//printf("3\n");
		cell.Ev[i]=-cell.Xi[i]-cell.Eg[i];
		//cell.Fn[i]=-cell.Xi[i]-cell.Eg[i]-kb*300.0*log();
		cell.Fp[i]=cell.Fp[i];
		//printf("4\n");
		cell.mun[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_mue);
		//printf("5\n");
		cell.mup[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_muh);
		cell.mn[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_me);
		cell.mp[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_mh);
		cell.Nc[i]=2.0*pow(((2.0*3.1415926*kb*cell.Tl[i]*cell.mn[i]*m0)/(hp*hp)),1.5);
		//printf("%e %e",pow(2.5e25/2.0,2.0/3.0)*hp*hp/2/pi/kb/300.0/m0,2.0*pow(((2.0*3.1415926*kb*cell.Tl[i]*9.974932e-01*m0)/(hp*hp)),1.5));
		//exit(0);
	//printf("%e %e %e %e %e\n",cell.Nc[i],cell.mn[i],cell.Tl[i],hp,m0);
//getchar();
		cell.Nv[i]=2.0*pow(((2.0*3.1415926*kb*cell.Tl[i]*cell.mp[i]*m0)/(hp*hp)),1.5);
		cell.ni[i]=sqrt(cell.Nc[i]*cell.Nv[i])*exp(-cell.Eg[i]/(2.0*cell.kTq));
		cell.kf[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kf);
		cell.a[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_a);
		//printf("%lf\n",cell.Nv[i]);

		//printf("6\n");

	}
//printf("ni=%e %e %e %e %e\n",cell.ni[0],cell.Nc[0],cell.Nv[0],exp(-cell.Eg[0]*q/(2.0*300.0*kb)),cell.Eg[0]);
//getchar();
//printf("Rod=here\n");

	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.phi[i]=0.0;
		cell.R[i]=0.0;//1e6;
		cell.n[i]=0.0;
	}

//device_dump(&cell);
//printf("Solved pos\n");
//getchar();
//initial guess

update(&cell);
double Vstop=cell.Vstop;
double V=cell.Vstart;
double Vstep=cell.Vstep;
cell.Vapplied=0.0;
get_initial(&cell);
solve_pos(&cell);
device_set_norm(&cell);
int IV=TRUE;
if (IV==TRUE)
{
FILE *iv;
iv=fopen("./iv.dat","w");
fclose(iv);
do
{
	cell.Vapplied=V;


	solve_cur(&cell);
	printf("V=%lf current = %e mA (%e A/m^2)\n",V,get_I(&cell)/1e-3,get_J(&cell));

	//printf("Initial system solved\n");
	//exit(0);

	iv=fopen("./iv.dat","a");
	fprintf(iv,"%e %e %e\n",cell.Vapplied,get_I(&cell),get_J(&cell));
	fclose(iv);
	//getchar();
	V+=Vstep;
	//printf("Plot\n");
	//getchar();

}while(V<Vstop);

}else
{
	cell.Vapplied=0.2;
	solve_cur(&cell);
	printf("V=%lf current = %e mA, J= %e A/m^2\n",cell.Vapplied,get_I(&cell)/1e-3,get_J(&cell));
}
device_free(&cell);
plot_close(&cell);
//plot_close(&cell);
exit(0);
//device_dump(&cell);


return 0;
}

