// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

int update2=FALSE;
void update_solver_vars(struct device *in,int clamp)
{
int i;
	for (i=0;i<in->ymeshpoints;i++)
	{
		double update=in->b[i];

		if (clamp==TRUE)
		{
			in->phi[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->phi[i]+=update;
		}

		update=(in->b[i+in->ymeshpoints]);
		//printf("%e\n",b[i+in->ymeshpoints]);
		if (clamp==TRUE)
		{
			in->x[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->x[i]+=update;
		}

		update=(in->b[i+2*in->ymeshpoints]);
		if (clamp==TRUE)
		{
			in->xp[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->xp[i]+=update;

		}

		update=(in->b[i+3*in->ymeshpoints]);
		if (clamp==TRUE)
		{
			in->ex[i]+=update/(1.0+fabs(update/in->electrical_clamp/(300.0*kb/q)));
		}else
		{
			in->ex[i]+=update;

		}
		//in->ex[i]+=in->b[i+3*in->ymeshpoints];


		in->Fn[i]=in->x[i]-in->phi[i];
		in->Fp[i]=-in->xp[i]-in->phi[i];



		in->Ec[i]=-in->phi[i]-in->Xi[i];
		in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];

		in->Nc[i]=2.0*pow(((2.0*3.1415926*kb*in->Tl[i]*in->mn[i]*m0)/(hp*hp)),1.5);
		in->Nv[i]=2.0*pow(((2.0*3.1415926*kb*in->Tl[i]*in->mp[i]*m0)/(hp*hp)),1.5);
		in->dn[i]=q*in->Nc[i]*exp((q*(in->x[i]+in->t[i]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);
		in->n[i]=in->Nc[i]*exp(((in->x[i]+in->t[i])*q)/(kb*in->Tl[i]));
		in->dp[i]=q*in->Nv[i]*exp((q*(in->xp[i]-in->tp[i]))/(kb*in->Tl[i]))/(kb*in->Tl[i]);
		in->p[i]=in->Nv[i]*exp(((in->xp[i]-in->tp[i])*q)/(kb*in->Tl[i]));

		//printf("%e update=%e\n",in->Fn[i],update/(1.0+fabs(update/0.1/(300.0*kb/q))));
	}

}

void fill_matrix(struct device *in)
{
double phil;
double phic;
double phir;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double ddh;
int pos=0;


double xil=0.0;
double xir=0.0;
double xipl=0.0;
double xipr=0.0;

double Ecl=0.0;
double Ecr=0.0;
double Ecc=0.0;
double Evl=0.0;
double Evr=0.0;
double Evc=0.0;

double Tll=0.0;
double Tlc=0.0;
double Tlr=0.0;
double Jnl=0.0;
double Jnr=0.0;
double Jpl=0.0;
double Jpr=0.0;

double Dnl=0.0;
double Dnc=0.0;
double Dnr=0.0;
double Dpl=0.0;
double Dpc=0.0;
double Dpr=0.0;
double nl=0.0;
double nc=0.0;
double nr=0.0;
double pl=0.0;
double pc=0.0;
double pr=0.0;
double ni=0.0;

double dJdxil=0.0;
double dJdxic=0.0;
double dJdxir=0.0;

//double dJhdxil=0.0;
//double dJhdxic=0.0;
//double dJhdxir=0.0;


double dJdphil=0.0;
double dJdphic=0.0;
double dJdphir=0.0;

//double dJhdphil=0.0;
//double dJhdphic=0.0;
//double dJhdphir=0.0;

double dJpdxipl=0.0;
double dJpdxipc=0.0;
double dJpdxipr=0.0;

double dnl=0.0;
double dnc=0.0;
double dnr=0.0;

double dpl=0.0;
double dpc=0.0;
double dpr=0.0;

double dJpdphil=0.0;
double dJpdphic=0.0;
double dJpdphir=0.0;


double dphidxic=0.0;
double dphidxipc=0.0;

double xnr;
double tnr;
double xnl;
double tnl;

double xpr;
double tpr;
double xpl;
double tpl;


double exl;
double exr;
double exc;
double Dexl;
double Dexc;
double Dexr;
double R;

double munl;
double munc;
double munr;

double mupl;
double mupc;
double mupr;

double epr;
double epc;
double epl;

double dexl;
double dexc;
double dexr;
double dex;
double G;
double Gex;
double F;
int i;
double dJdxipc=0.0;
double dJpdxic=0.0;
	//update(in);
		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{

			if (i==0)
			{
				exl=0.0;
				Dexl=in->Dex[0];
				phil=in->Vapplied;
				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
				Tll=in->Tl[0];
				Dnl=in->Dn[0];
				Dpl=in->Dp[0];

				Ecl=-in->Xi[0]-phil;
				Evl=-in->Xi[0]-phil-in->Eg[0];		//Hier bin ich.
				//printf("a=%e %e %e %e\n",in->Xi[0],phil,in->Eg[0],in->Vapplied);
				//getchar();
				epl=in->epsilonr[0]*epsilon0;
				//printf("%e %e %e\n",in->Xi[0],phil,in->Vapplied);
				xnl=in->Fi[0];
				tnl=in->Xi[0];
				nl=in->Nc[i]*exp((q*(xnl+tnl))/(kb*in->Tl[i]));
				dnl=(q/(kb*in->Tl[i]))*in->Nc[i]*exp((q*(xnl+tnl))/(kb*in->Tl[i]));

				xpl=-in->Fi[0];
				tpl=in->Xi[0]+in->Eg[0];
				pl=in->Nv[i]*exp((q*(xpl-tpl))/(kb*in->Tl[i]));
				dpl=(q/(kb*in->Tl[i]))*in->Nv[i]*exp((q*(xpl-tpl))/(kb*in->Tl[i]));
				munl=in->mun[0];
				mupl=in->mup[0];


				//dnl=q*in->Nc[i]/(kb*in->Tl[i]);

			}else
			{
				Dexl=in->Dex[i-1];
				exl=in->ex[i-1];
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
				Tll=in->Tl[i-1];
				Dnl=in->Dn[i-1];
				Dpl=in->Dn[i-1];
				Ecl=in->Ec[i-1];
				Evl=in->Ev[i-1];
				nl=in->n[i-1];
				dnl=in->dn[i-1];
				pl=in->p[i-1];
				dpl=in->dp[i-1];
				munl=in->mun[i-1];
				mupl=in->mup[i-1];
				epl=in->epsilonr[i-1]*epsilon0;


			}

			Ecc=-in->Xi[i]-in->phi[i];
			Evc=-in->Xi[i]-in->phi[i]-in->Eg[i];

			if (i==(in->ymeshpoints-1))
			{
				Dexr=in->Dex[i];
				exr=0.0;
				phir=in->Vr;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
				Tlr=in->Tl[i];
				Dnr=in->Dn[i];
				Dpr=in->Dp[i];

				Ecr=-in->Xi[i]-phir;
				Evr=-in->Xi[i]-phir-in->Eg[i];



				xnr=phir+in->Fi[i];
				tnr=in->Xi[i];	//This looks very odd
				nr=in->Nc[i]*exp((q*(xnr+tnr))/(kb*in->Tl[i]));

				dnr=(q/(kb*in->Tl[i]))*in->Nc[i]*exp((q*(xnr+tnr))/(kb*in->Tl[i]));

				xpr=-(phir+in->Fi[i]);
				tpr=in->Xi[i]+in->Eg[i];	//This looks very odd
				pr=in->Nv[i]*exp((q*(xpr-tpr))/(kb*in->Tl[i]));
				dpr=(q/(kb*in->Tl[i]))*in->Nv[i]*exp((q*(xpr-tpr))/(kb*in->Tl[i]));

				munr=in->mun[i];
				mupr=in->mup[i];
				epr=in->epsilonr[i]*epsilon0;

			}else
			{
				Dexr=in->Dex[i+1];
				exr=in->ex[i+1];
				phir=in->phi[i+1];
				yr=in->ymesh[i+1];
				Tlr=in->Tl[i+1];
				Dnr=in->Dn[i+1];
				Dpr=in->Dp[i+1];

				Ecr=in->Ec[i+1];
				Evr=in->Ev[i+1];

				nr=in->n[i+1];
				dnr=in->dn[i+1];
				pr=in->p[i+1];
				dpr=in->dp[i+1];
				munr=in->mun[i+1];
				mupr=in->mup[i+1];
				epr=in->epsilonr[i+1]*epsilon0;

			}


		/*	if (i>(in->ymeshpoints-10))
			{
				printf("testrod=%.20e\n",Evc);
				getchar();
			}*/

			munc=in->mun[i];
			mupc=in->mup[i];

			munl=(munl+munc)/2.0;
			munr=(munr+munc)/2.0;

			mupl=(mupl+mupc)/2.0;
			mupr=(mupr+mupc)/2.0;

			epc=in->epsilonr[i]*epsilon0;


			exc=in->ex[i];
			Dexc=in->Dex[i];
			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			ddh=(dyl+dyr)/2.0;
			phic=in->phi[i];
			Tlc=in->Tl[i];


			Dnc=in->Dn[i];
			Dnl=(Dnl+Dnc)/2.0;
			Dnr=(Dnr+Dnc)/2.0;

			Dpc=in->Dp[i];
			Dpl=(Dpl+Dpc)/2.0;
			Dpr=(Dpr+Dpc)/2.0;


			nc=in->n[i];
			pc=in->p[i];
			ni=in->ni[i];

			//printf("n=%e\n",nc);
			dnc=in->dn[i];
			dpc=in->dp[i];

			xil=q*(Ecc-Ecl)/(kb*Tlc);
			xir=q*(Ecr-Ecc)/(kb*Tlc);

			F=(phir-phil)/(dyr+dyl);

R=in->R[i];
Gex=in->Gex[i];
//printf("%e",G);
//getchar();

//	double dphil=-epl/dyl/ddh; //epsilon*(2.0*dyr)/((dyl+dyr)*(dyl*dyr));
//	double dphic=epr/dyr/ddh+epl/dyl/ddh;
//	double dphir=-epr/dyr/ddh;

	double e0=(epl+epc)/2.0;
	double e1=(epc+epr)/2.0;

	double dphil=-e0/dyl/ddh;
	double dphic=e0/dyl/ddh+e1/dyr/ddh;
	double dphir=-e1/dyr/ddh;
	double Rbi=0.0;
	double deriv=phil*dphil+phic*dphic+phir*dphir;
//if (i>45)
//{
//	printf("pos = %d %e %e %e %e\n",i,deriv,phil,phic,phir);
//	getchar();
//}

	//printf("poisson=%e %e %e\n",dphil,dphic,dphir);
	//dphic+=-q*(dpc-dnc);
//-q*(in->p[i]-in->n[i]-in->Nad[i])


	dphidxic=q*(dnc);
	dphidxipc=-q*(dpc);
	dphidxic/=in->n0*q;
	dphidxipc/=in->n0*q;

	//printf("deriv=%e %e\n",dphidxic,dphidxipc);
	//getchar();
	double kdiss=0.0;
	double mu_min=0.0;
	mu_min=(in->mup[i]+in->mun[i])/2.0;

	double langevin=q*(mu_min)/(epc);
	//printf("%e \n",langevin);
	//getchar();
	in->Rbi[i]=langevin*(nc*pc-ni*ni);
	Rbi=in->Rbi[i];
	//printf("%e",Rbi);
	//getchar();
	//printf("%e %e %e %e\n",Rbi,nc,pc,ni);
	double b=q*q*q*F/(8.0*pi*epc*kb*kb*Tlc*Tlc);
	double a=in->a[i];
	//printf("%e\n",Rbi);
	//getchar();
	double Eb=q*q/(4.0*pi*epc*a);
	double dexdn=0.0;
	double dexdp=0.0;

	double dJndex=0.0;
	double dJpdex=0.0;

	double kf=in->kf[i];

	double dRbidxic=-langevin*(dnc*pc);
	double dRbidxipc=-langevin*(nc*dpc);
	double dRdisdn=0.0;
	double dRdisdp=0.0;
	double Rdis=0.0;
	//if (in->Vapplied>0.2)
	//{
		//in->Gp_on=TRUE;
	//}

	//G=1e24;
	if (in->enable_ex==TRUE)
	{
		kdiss=3.0*Rbi*(exp(-Eb/(Tlc*kb))*(1.0+b+b*b/3.0))/(4.0*pi*a*a*a);
		Rdis=kdiss*exc;
		in->G[i]=Rdis;
		dRdisdn=3.0*langevin*exc*dnc*pc*(exp(-Eb/(Tlc*kb))*(1.0+b+b*b/3.0))/(4.0*pi*a*a*a);
		dRdisdp=3.0*langevin*exc*nc*dpc*(exp(-Eb/(Tlc*kb))*(1.0+b+b*b/3.0))/(4.0*pi*a*a*a);
		dexdn=-dRdisdn;
		dexdp=-dRdisdp;
		//printf("%e %e\n",kdiss,Rbi);
		//getchar();
		dexl=Dexc/dyl/(dyl+dyr)/2.0;;
		dexc=-Dexc*(1.0/dyl+1.0/dyr)/(dyl+dyr)/2.0;
		dexr=Dexc/dyr/(dyl+dyr)/2.0;

		dex=dexl*exl+dexc*exc+dexr*exr+Gex-Rdis-kf*exc;


		//do
		G=in->G[i];
		//in->Gp_on=FALSE;
		//if (in->Vapplied>0.2) in->Gn_on=TRUE;
		if (in->Gp_on==TRUE)
		{
			dJpdex+=kdiss;
		}

		if (in->Gn_on==TRUE)
		{
			dJndex+=kdiss;
		}


		//don't do
		/*dRdisdn=0.0;
		dRdisdp=0.0;
		G=0.0;
		dJndex=0.0;
		dJpdex=0.0;*/
		//}
		//printf("%e %e\n",kdiss*exc,Rbi);
		dexc+=-kdiss-kf;
	}

	if (in->Rbi_on==FALSE) Rbi=0.0;

//Rbi=0.0;
//G=0.0;

	/*if (in->Vapplied<0.3)
	{

	}else
	{
		printf("On\n");
	}*/


//Normalization
//<clean>
int norm=TRUE;

if (norm==TRUE)
{
	//Distance
	dyl/=in->x0;
	dyr/=in->x0;
	ddh/=in->x0;
	Rbi/=in->R0;
	dRbidxic/=in->R0;
	dRbidxipc/=in->R0;
	dRdisdn/=in->R0;
	dRdisdp/=in->R0;

	//carriers
	nl/=in->n0;nc/=in->n0;nr/=in->n0;
	pl/=in->n0;pc/=in->n0;pr/=in->n0;

	dnl/=in->n0;dnc/=in->n0;dnr/=in->n0;
	dpl/=in->n0;dpc/=in->n0;dpr/=in->n0;

	//Recomb
	R/=in->R0;
	G/=in->R0;

	dJndex/=in->R0;
	dJpdex/=in->R0;
	//D0
	Dnl/=in->D0;
	Dnr/=in->D0;
	Dpr/=in->D0;
	Dpl/=in->D0;

	munl/=in->mu0;
	munc/=in->mu0;
	munr/=in->mu0;

	mupl/=in->mu0;
	mupc/=in->mu0;
	mupr/=in->mu0;

	dphil/=in->n0*q;
	dphic/=in->n0*q;
	dphir/=in->n0*q;

	dexl/=in->Dex0;
	dexc/=in->Dex0;
	dexr/=in->Dex0;
	dexdn/=in->Dex0;
	dexdp/=in->Dex0;
	dex/=in->Dex0;
	//</clean>

}


			//printf("%e %e %e %e  \n",Ecl,Ecc,-in->Xi[i],-phil);
			xipl=q*(Evc-Evl)/(kb*Tlc);
			xipr=q*(Evr-Evc)/(kb*Tlc);

			//printf("rod=%le %le %le\n",xipl,Evl,Evc);
			//getchar();

			//double dphidn=in->dn[i];
			//double dphidp=in->dp[i];

			Jnl=(Dnl/dyl)*(B(-xil)*nc-B(xil)*nl);
			Jnr=(Dnr/dyr)*(B(-xir)*nr-B(xir)*nc);

			//printf("%e %e\n",dyl,dyr);
			//getchar();
			Jpr=(Dpr/dyr)*(B(-xipr)*pc-B(xipr)*pr);
			Jpl=(Dpl/dyl)*(B(-xipl)*pl-B(xipl)*pc);

			/*if (i>(in->ymeshpoints-5))
			{
				printf("%e %e %e\n",Evr/in->kTq,Evc/in->kTq,(Evr-Evc)/in->kTq);
				getchar();
			if (i==in->ymeshpoints-1) exit(0);
			}*/

			in->Jn[i]=(Jnl+Jnr)*in->J0/2.0;
			in->Jp[i]=(Jpl+Jpr)*in->J0/2.0;


			dJdxil=(Dnl/dyl/ddh)*(B(xil)*dnl);
			dJdxic=-(Dnl/dyl)*B(-xil)*dnc/ddh-(Dnr/dyr)*B(xir)*dnc/ddh;
			dJdxir=(Dnr/dyr)*B(-xir)*dnr/ddh;

			dJdxipc=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJdxic+=dRbidxic;
				dJdxipc+=dRbidxipc;
			}

			if (in->Gn_on==TRUE)
			{
				dJdxic+=dRdisdn;
				dJdxipc+=dRdisdp;
			}
//printf("%e\n",-langevin*(dnc*pc));
//getchar();

			dJpdxipl=-(Dpl/dyl/ddh)*(B(-xipl)*dpl);//needs sorting out
			dJpdxipc=((Dpl/dyl/ddh)*B(xipl)*dpc+(Dpr/dyr/ddh)*B(-xipr)*dpc);
			dJpdxipr=-(Dpr/dyr/ddh)*B(xipr)*dpr;



			dJpdxic=0.0;
			if (in->Rbi_on==TRUE)
			{
				dJpdxipc+=dRbidxipc;
				dJpdxic+=dRbidxic;
			}

			if (in->Gp_on==TRUE)
			{
				dJpdxipc+=dRdisdp;
				dJpdxic+=dRdisdn;
			}

			//<clean>
			double ktq=in->kTq;
			dJdphil=(munl/dyl/ddh)*(dB(-xil)*nc+dB(xil)*nl)/ktq;
			dJdphic=-((munl/dyl)*(dB(-xil)*nc+dB(xil)*nl)/ddh+(munr/dyr)*(dB(-xir)*nr+dB(xir)*nc)/ddh)/ktq;
			dJdphir=(munr/dyr/ddh)*(dB(-xir)*nr+dB(xir)*nc)/ktq; //Is there a Q and


			//</clean>


			dJpdphil=((mupl)/dyl/ddh)*(dB(-xipl)*pl+dB(xipl)*pc)/ktq;
			dJpdphic=-(((mupl)/dyl/ddh)*(dB(-xipl)*pl+dB(xipl)*pc)+(mupr/dyr/ddh)*(dB(-xipr)*pc+dB(xipr)*pr))/ktq;
			dJpdphir=(mupr/dyr/ddh)*(dB(-xipr)*pc+dB(xipr)*pr)/ktq; //Is there a Q and

			//printf("%d %e %e\n",i,nc,pl);
			//getchar();
			//printf("%e %e %e\n",dJpdphil,dJpdphic,dJpdphir);
			if (i!=0)
			{
				in->Ti[pos]=i;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dphil;
				pos++;
				//electron				ttttt
				in->Ti[pos]=in->ymeshpoints+i;
				in->Tj[pos]=in->ymeshpoints+i-1;
				in->Tx[pos]=dJdxil;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dJdphil;
				pos++;

				//hole
				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i+2*in->ymeshpoints-1;
				in->Tx[pos]=dJpdxipl;
				pos++;

				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i-1;
				in->Tx[pos]=dJpdphil;
				pos++;

				if (in->enable_ex==TRUE)
				{
					//exiton
					in->Ti[pos]=i+3*in->ymeshpoints;
					in->Tj[pos]=i+3*in->ymeshpoints-1;
					in->Tx[pos]=dexl;
					pos++;

				}
			}

			//phi
			in->Ti[pos]=i;
			in->Tj[pos]=i;
			in->Tx[pos]=dphic;
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+in->ymeshpoints;
			in->Tx[pos]=dphidxic;
			pos++;

			in->Ti[pos]=i;
			in->Tj[pos]=i+2*in->ymeshpoints;
			in->Tx[pos]=dphidxipc;
			pos++;

			//electron
			in->Ti[pos]=in->ymeshpoints+i;
			in->Tj[pos]=in->ymeshpoints+i;
			in->Tx[pos]=dJdxic;
			pos++;


			in->Ti[pos]=i+in->ymeshpoints;
			in->Tj[pos]=i;
			in->Tx[pos]=dJdphic;
			pos++;

			in->Ti[pos]=i+in->ymeshpoints;
			in->Tj[pos]=2*in->ymeshpoints+i;
			in->Tx[pos]=dJdxipc;
			pos++;

			//hole
			in->Ti[pos]=2*in->ymeshpoints+i;
			in->Tj[pos]=2*in->ymeshpoints+i;
			in->Tx[pos]=dJpdxipc;
			pos++;


			in->Ti[pos]=i+2*in->ymeshpoints;
			in->Tj[pos]=i;
			in->Tx[pos]=dJpdphic;
			pos++;

			in->Ti[pos]=2*in->ymeshpoints+i;
			in->Tj[pos]=in->ymeshpoints+i;
			in->Tx[pos]=dJpdxic;
			pos++;

			//exciton

			if (in->enable_ex==TRUE)
			{
				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dexc;
				pos++;

				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+in->ymeshpoints;
				in->Tx[pos]=dexdn;
				pos++;

				in->Ti[pos]=i+3*in->ymeshpoints;
				in->Tj[pos]=i+2*in->ymeshpoints;
				in->Tx[pos]=dexdp;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dJndex;
				pos++;

				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i+3*in->ymeshpoints;
				in->Tx[pos]=dJpdex;
				pos++;

			}

			if (i!=(in->ymeshpoints-1))
			{
				//phi
				in->Ti[pos]=i;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dphir;
				pos++;

				//electron
				in->Ti[pos]=in->ymeshpoints+i;
				in->Tj[pos]=in->ymeshpoints+i+1;
				in->Tx[pos]=dJdxir;
				pos++;

				in->Ti[pos]=i+in->ymeshpoints;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJdphir;
				pos++;

				//hole
				in->Ti[pos]=2*in->ymeshpoints+i;
				in->Tj[pos]=2*in->ymeshpoints+i+1;
				in->Tx[pos]=dJpdxipr;
				pos++;

				in->Ti[pos]=i+2*in->ymeshpoints;
				in->Tj[pos]=i+1;
				in->Tx[pos]=dJpdphir;
				pos++;


				//exiton
				if (in->enable_ex==TRUE)
				{
					in->Ti[pos]=i+3*in->ymeshpoints;
					in->Tj[pos]=i+3*in->ymeshpoints+1;
					in->Tx[pos]=dexr;
					pos++;
				}
			}

			in->b[i]=-(deriv-q*(in->p[i]-in->n[i]-in->Nad[i]));
			in->b[i]/=in->n0*q;

			//printf("%e %e %e %e\n",Jnl,Jnr,ddh,in->R[i]);

			in->b[in->ymeshpoints+i]=-((Jnr-Jnl)/ddh-Rbi);
			if (in->Gn_on) in->b[in->ymeshpoints+i]-=(G);
			in->b[2*in->ymeshpoints+i]=-((Jpr-Jpl)/ddh-Rbi); //+
			if (in->Gp_on) in->b[2*in->ymeshpoints+i]-=(G);
			printf("%e %e\n",1e-12*in->R0,Rbi*in->R0);



			//printf("%e %e %e\n",(Jpr-Jpl)/ddh,Rbi,G);

			if (in->enable_ex==TRUE)
			{

				in->b[3*in->ymeshpoints+i]=-(dex);
			}



		if (pos>in->N)
		{
			printf("Error\n");
		}
		}

}

double get_cur_error(struct device *in)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(in->b[i]);
	tot+=mod(in->b[in->ymeshpoints+i]);
	tot+=+mod(in->b[2*in->ymeshpoints+i]);

	if (in->enable_ex==TRUE)
	{
		tot+=+mod(in->b[3*in->ymeshpoints+i]);
	}
}
return tot;
}

void solver_get_memory(struct device *in)
{


in->N=in->ymeshpoints*3-2;	//Possion main
in->N+=in->ymeshpoints*3-2;	//Je main
in->N+=in->ymeshpoints*3-2;	//Jh main
//printf("%d",in->enable_ex);
//getchar();
if (in->enable_ex==TRUE)
{
in->N+=in->ymeshpoints*3-2;	//ex main
in->N+=in->ymeshpoints;		//ex dn
in->N+=in->ymeshpoints;		//ex dp
in->N+=in->ymeshpoints;		//dJndex
in->N+=in->ymeshpoints;		//dJpdex

}

in->N+=in->ymeshpoints*3-2;	//dJe/phi
in->N+=in->ymeshpoints*3-2;	//dJh/phi

in->N+=in->ymeshpoints;		//dphi/dJe
in->N+=in->ymeshpoints;		//dphi/dJh

in->N+=in->ymeshpoints;		//dJndp
in->N+=in->ymeshpoints;		//dJpdn

in->M=in->ymeshpoints;	//Pos
in->M+=in->ymeshpoints;		//Je
in->M+=in->ymeshpoints;		//Jh
if (in->enable_ex==TRUE)
{
in->M+=in->ymeshpoints;		//ex
}

in->Ti =    malloc(in->N*sizeof(int));
in->Tj =    malloc(in->N*sizeof(int));
in->Tx = malloc(in->N*sizeof(double));
in->b = malloc(in->M*sizeof(double));
in->back_b = malloc(in->M*sizeof(double));


}

void dump_for_plot(struct device *in)
{
int i;
FILE *out =fopen ("./band.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e %e %e %e %e\n",in->ymesh[i],in->Ec[i],in->Ev[i],in->Fi[i],in->Fn[i],in->Fp[i]);
}

fclose(out);

out =fopen ("./j.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e %e %e\n",in->ymesh[i],in->Jn[i],in->Jp[i],in->Jn[i]+in->Jp[i]);
}

fclose(out);

out =fopen ("./n.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e %e\n",in->ymesh[i],in->n[i],in->p[i]);
}

fclose(out);


out =fopen ("./Dex.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e %e\n",in->ymesh[i],in->ex[i],in->G[i]);
}

fclose(out);

out =fopen ("./R.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e\n",in->ymesh[i],in->Rbi[i]);
}

fclose(out);

}

int solve_cur(struct device *in)
{
double error;
int ittr=0;
printf("Solve cur\n");



/*	int r;
	FILE *outa=fopen("./out.dat","w");
	for (r=0;r<in->ymeshpoints;r++)
	{
		fprintf(outa,"%e %e\n",in->ymesh[r],in->phi[r]);

	}
	fclose(outa);
	exit(0);*/
int doback=FALSE;
//in->enable_back=FALSE;
int keep_going=FALSE;
	do
	{

		update2=FALSE;
		if (ittr==0) update2=TRUE;
		fill_matrix(in);
		dump_for_plot(in);
		plot_now(in);
		solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
		getchar();
	//dRbidxic=0.0;
	//dRbidxipc=0.0;
	//Rbi=0.0;
	//G=0.0;//in->G[i];
		//printf("Want to solve\n");
	//	getchar();
		//solver_dump_matrix(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
		//solver_cmp_matrix(M,N,Ti,Tj, Tx,b);
		//getchar();

		if (in->enable_back)
		{
			if (ittr<5)
			{
				doback=TRUE;
			}else
			{
				doback=FALSE;
			}
		}
		doback=FALSE;

		//if (error<1e-6) doback=TRUE;

		if (ittr<5) doback=FALSE;
		//in->enable_back=FALSE;

		if (doback==TRUE)
		{
			do_backtrack(in);
			update_solver_vars(in,FALSE);
		}else
		{
			solver(in->M,in->N,in->Ti,in->Tj, in->Tx,in->b);
			update_solver_vars(in,TRUE);
		}
		//update(in);
//getchar();
		error=get_cur_error(in);
		printf("%d Cur error = %e",ittr,error);
		if (doback==TRUE)
		{
			printf(" b");
		}

		printf("\n");
		ittr++;

		in->converge=fopen("./converge.dat","a");
		fprintf(in->converge,"%e\n",error);
		fclose(in->converge);

		keep_going=FALSE;

		//printf("%le\n",in->min_cur_error);
		//getchar();
		if ((ittr<in->max_electrical_itt)&&(error>in->min_cur_error)) keep_going=TRUE;
	}while(keep_going==TRUE);

return 0;
}

