// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
double min_pos_error=1e-14;


double get_p_error(struct device *in,double *b)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(b[i]);
}
return tot;
}


int solve_pos(struct device *in)
{
printf("Solve pos\n");
int i;

int N=in->ymeshpoints*3-2;


int M=in->ymeshpoints;
int *Ti =    malloc(N*sizeof(int));
int *Tj =    malloc(N*sizeof(int));
double *Tx = malloc(N*sizeof(double));
double *b = malloc(M*sizeof(double));


double phil;
double phic;
double phir;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double dyc=0.0;
int ittr=0;
int pos=0;
double error;
double el=0.0;
double ec=0.0;
double er=0.0;
double e0=0.0;
double e1=0.0;


	do
	{

		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{


			if (i==0)
			{
				phil=in->Vl;
				el=in->epsilonr[0]*epsilon0;
				//printf("Vl=%e\n",in->Vl);
				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);


			}else
			{
				el=in->epsilonr[i-1]*epsilon0;
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
			}

			if (i==(in->ymeshpoints-1))
			{
				phir=in->Vr;
				//printf("Vr=%e\n",in->Vr);
				er=in->epsilonr[i]*epsilon0;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
			}else
			{
				er=in->epsilonr[i+1]*epsilon0;
				phir=in->phi[i+1];
				yr=in->ymesh[i+1];

			}


			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			dyc=(dyl+dyr)/2.0;

			ec=in->epsilonr[i]*epsilon0;
			e0=(el+ec)/2.0;
			e1=(ec+er)/2.0;
	//printf("phi=%e",in->phi[i]);
	//getchar();
			phic=in->phi[i];

			double pdhidn=(q/(kb*in->Tl[i]))*in->Nc[i]*exp(((in->Fi[i]-(-in->phi[i]-in->Xi[i])))/(in->kTq));
			double dphidp=-(q/(kb*in->Tl[i]))*in->Nv[i]*exp((((-in->phi[i]-in->Xi[i]-in->Eg[i])-in->Fi[i]))/(in->kTq));
			//pdhidn=0.0;
			//dphidp=0.0;
			double dphil=-e0/dyl/dyc;
			double dphic=e0/dyl/dyc+e1/dyr/dyc;
			double dphir=-e1/dyr/dyc;

			double dphi=dphil*phil+dphic*phic+dphir*phir;

			//printf("%e\n",q/(kb*in->Tl[i]));
			//getchar();

			dphic+=-q*(pdhidn-dphidp);
			//printf("%e \n",-q*(pdhidn-dphidp));
			//getchar();
			//if (i==(in->ymeshpoints-1))
			//{
			//	dphic+=dphir;
			//	dphir=0.0;
			//}

			if (i!=0)
			{
				Ti[pos]=i;
				Tj[pos]=i-1;
				Tx[pos]=dphil;
				pos++;



			}

			Ti[pos]=i;
			Tj[pos]=i;
			Tx[pos]=dphic;



			pos++;



			if (i!=(in->ymeshpoints-1))
			{
				Ti[pos]=i;
				Tj[pos]=i+1;
				Tx[pos]=dphir;
				pos++;

			}

			b[i]=-(dphi-q*(in->n[i]-in->p[i]+in->Nad[i]));
			in->n[i]=in->Nc[i]*exp(((in->Fi[i]-in->Ec[i])*q)/(kb*in->Tl[i]));
			//if (i==80)
			//{

				//getchar();
			//}


		}

		//solver_dump_matrix(M,N,Ti,Tj, Tx,b);
		//getchar();
		//exit(0);
		solver(M,N,Ti,Tj, Tx,b);


		for (i=0;i<in->ymeshpoints;i++)
		{
			in->phi[i]+=b[i];///(1.0+fabs(b[i]/10.0/(300.0*kb/q)));
		}

		for (i=0;i<in->ymeshpoints;i++)
		{
			in->Ec[i]=-in->phi[i]-in->Xi[i];
			in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];
			in->n[i]=in->Nc[i]*exp(((in->Fi[i]-in->Ec[i])*q)/(kb*in->Tl[i]));
			in->dn[i]=(q/(kb*in->Tl[i]))*in->Nc[i]*exp((q*(in->Fi[i]-in->Ec[i]))/(kb*in->Tl[i]));
			in->Fn[i]=in->Fi[i];
			in->Fp[i]=in->Fi[i];
			in->x[i]=in->phi[i]+in->Fn[i];
			in->t[i]=in->Xi[i];
			in->xp[i]=-(in->phi[i]+in->Fp[i]);
			in->tp[i]=in->Xi[i]+in->Eg[i];
			in->p[i]=in->Nv[i]*exp(((in->xp[i]-in->tp[i])*q)/(kb*in->Tl[i]));
			in->dp[i]=(q/(kb*in->Tl[i]))*in->Nv[i]*exp(((in->xp[i]-in->tp[i])*q)/(kb*in->Tl[i]));


		}
		error=get_p_error(in,b);
		printf("Pos error = %e\n",error);
		ittr++;
	//getchar();
	}while((ittr<20)&&(error>min_pos_error));

FILE *out;
out=fopen("./start.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
	fprintf(out,"%e %e %e %e %e %e %e\n",in->ymesh[i],in->Ec[i],in->Ev[i],in->Fi[i],in->Fn[i],in->Fp[i],in->phi[i]);
}
fclose(out);

out =fopen ("./dump1","w");
for (i=0;i<in->ymeshpoints;i++)
{
		fprintf(out,"%e %e %e\n",in->ymesh[i],in->phi[i],in->n[i]);
}

fclose(out);
//exit(0);
free(Ti);
free(Tj);
free(Tx);
free(b);
return 0;
}

